package org.openxma.dsl.generator.mojo;

import static org.openxma.dsl.common.DslConstants.OPENXMA_JSF_POM_FILE_EXTENSION;
import static org.openxma.dsl.common.extensions.EObjectPropertiesAdapter.setProperty;
import static org.openxma.dsl.generator.GeneratorConstants.PRESENTATION_MODEL_GENERATOR;
import static org.openxma.dsl.generator.GeneratorConstants.PRESENTATION_MODEL_GENERATOR_JSF;

import java.io.File;

import org.apache.maven.plugin.MojoExecutionException;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.mwe.core.WorkflowContext;
import org.eclipse.emf.mwe.core.WorkflowContextDefaultImpl;
import org.eclipse.emf.mwe.core.issues.IssuesImpl;
import org.eclipse.emf.mwe.core.monitor.NullProgressMonitor;
import org.eclipse.emf.mwe.utils.DirectoryCleaner;
import org.eclipse.xtext.ISetup;
import org.openxma.dsl.generator.DefaultGeneratorConfiguration;
import org.openxma.dsl.generator.DefaultGeneratorFactory;
import org.openxma.dsl.generator.Generator;

import com.google.inject.Provider;

/**
 * Maven plugin used to execute the openXMA jsfGenerator
 * 
 * @goal generate-jsf-pom
 * @requiresDependencyResolution compile
 * @phase generate-sources
 */
public class JsfGeneratorMojo extends PomGeneratorMojo {
    /**
     * @required
     * @parameter expression="${basedir}/src/main/webapp"
     */
    private File webAppDirectory;

    /**
     * @required
     * @parameter expression="${basedir}/src/main/webapp"
     */
    private File genWebAppDirectory;

    protected Generator createGenerator() throws MojoExecutionException {
        DefaultGeneratorConfiguration generatorConfiguration = createGeneratorConfiguration(OPENXMA_JSF_POM_FILE_EXTENSION);
        Generator generator = new DefaultGeneratorFactory().createAndInit(generatorConfiguration);
        return generator;
    }

    @Override
    protected DefaultGeneratorConfiguration createGeneratorConfiguration(String fileExtension)
            throws MojoExecutionException {
        DefaultGeneratorConfiguration generatorConfiguration = super.createGeneratorConfiguration(fileExtension);
        generatorConfiguration.setWebAppFolder(URI.createURI(webAppDirectory.toURI().toString()));
        generatorConfiguration.setGenWebAppFolder(URI.createURI(genWebAppDirectory.toURI().toString()));
        return generatorConfiguration;
    }

    protected void cleanDirectory() {
        if (cleanDirectory) {
            WorkflowContext workflowContext = new WorkflowContextDefaultImpl();
            IssuesImpl issues = new IssuesImpl();
            NullProgressMonitor progressMonitor = new NullProgressMonitor();
            DirectoryCleaner directoryCleaner = new DirectoryCleaner();
            directoryCleaner.setDirectory(generatedSourceDirectory.toString());
            directoryCleaner.invoke(workflowContext, progressMonitor, issues);
            directoryCleaner.setDirectory(generatedResourceDirectory.toString());
            directoryCleaner.invoke(workflowContext, progressMonitor, issues);
            if (genWebAppDirectory != null && !genWebAppDirectory.equals(webAppDirectory)) {
                directoryCleaner.setDirectory(genWebAppDirectory.toString());
                directoryCleaner.invoke(workflowContext, progressMonitor, issues);
            }
        }
    }

    @Override
    protected Provider<ResourceSet> createSharedResourceSet(final ISetup setup) {
        return new Provider<ResourceSet>() {

            public ResourceSet get() {
                ResourceSet resourceSet = setup.createInjectorAndDoEMFRegistration().getInstance(ResourceSet.class);
                setProperty(resourceSet, PRESENTATION_MODEL_GENERATOR, PRESENTATION_MODEL_GENERATOR_JSF);
                return resourceSet;
            }
        };
    }

}
